@@ -67,6 +67,7 @@ int i_fenum = 0;
 int i_canum = 0;
 char *psz_delsys = NULL;
 int i_frequency = 0;
+char *psz_lnb_type = "universal";
 int dvb_plp_id = 0;
 int i_inversion = -1;
 int i_srate = 27500000;
@@ -80,6 +81,10 @@ int i_bandwidth = 8;
 char *psz_modulation = NULL;
 int i_pilot = -1;
 int i_mis = 0;
+char *psz_mis_pls_mode = "ROOT";
+int i_mis_pls_mode = 0;
+int i_mis_pls_code = 0;
+int i_mis_is_id = 0;
 int i_fec_lp = 999;
 int i_guard = -1;
 int i_transmission = -1;
@@ -92,7 +97,7 @@ int b_select_pmts = 0;
 int b_random_tsid = 0;
 char *psz_udp_src = NULL;
 int i_asi_adapter = 0;
-const char *psz_native_charset = "UTF-8";
+const char *psz_native_charset = "UTF-8//IGNORE";
 print_type_t i_print_type = PRINT_TEXT;
 bool b_print_enabled = false;
 FILE *print_fh;
@@ -114,7 +119,7 @@ static mtime_t i_latency_global = DEFAULT_OUTPUT_LATENCY;
 static mtime_t i_retention_global = DEFAULT_MAX_RETENTION;
 static int i_ttl_global = 64;
 
-static const char *psz_dvb_charset = "UTF-8";
+static const char *psz_dvb_charset = "UTF-8//IGNORE";
 static iconv_t conf_iconv = (iconv_t)-1;
 static uint16_t i_network_id = 0xffff;
 static dvb_string_t network_name;
@@ -205,53 +210,70 @@ char *config_stropt( const char *psz_string )
     return ret;
 }
 
-static char *config_striconv( const char *psz_string )
+static uint8_t *config_striconv( const char *psz_string,
+                                 const char *psz_charset, size_t *pi_length )
 {
     char *psz_input = config_stropt(psz_string);
+    *pi_length = strlen(psz_input);
 
-    if ( !strcasecmp( psz_native_charset, psz_dvb_charset ) )
-        return psz_input;
+    /* do not convert ASCII strings */
+    const char *c = psz_string;
+    while (*c)
+        if (!isascii(*c++))
+            break;
+    if (!*c)
+        return (uint8_t *)psz_input;
+
+    if ( !strcasecmp( psz_native_charset, psz_charset ) )
+        return (uint8_t *)psz_input;
 
 #ifdef HAVE_ICONV
     if ( conf_iconv == (iconv_t)-1 )
     {
-        conf_iconv = iconv_open( psz_dvb_charset, psz_native_charset );
+        conf_iconv = iconv_open( psz_charset, psz_native_charset );
         if ( conf_iconv == (iconv_t)-1 )
-            return psz_input;
+            return (uint8_t *)psz_input;
     }
 
-    size_t i_input = strlen( psz_input );
+    char *psz_tmp = psz_input;
+    size_t i_input = *pi_length;
     size_t i_output = i_input * 6;
-    char *psz_output = malloc( i_output );
-    char *p = psz_output;
-    if ( iconv( conf_iconv, &psz_input, &i_input, &p, &i_output ) == -1 )
+    size_t i_available = i_output;
+    char *p_output = malloc( i_output );
+    char *p = p_output;
+    if ( iconv( conf_iconv, &psz_tmp, &i_input, &p, &i_available ) == -1 )
     {
-        free( psz_output );
-        return psz_input;
+        free( p_output );
+
+        return (uint8_t *)psz_input;
     }
 
     free(psz_input);
-    return psz_output;
+    *pi_length = i_output - i_available;
+    return (uint8_t *)p_output;
 #else
     msg_Warn( NULL,
               "unable to convert from %s to %s (iconv is not available)",
-              psz_native_charset, psz_dvb_charset );
-    return psz_input;
+              psz_native_charset, psz_charset );
+    return (uint8_t *)psz_input;
 #endif
 }
 
-static void config_strdvb( dvb_string_t *p_dvb_string, const char *psz_string )
+static void config_strdvb( dvb_string_t *p_dvb_string, const char *psz_string,
+                           const char *psz_charset )
 {
     if (psz_string == NULL)
     {
         dvb_string_init(p_dvb_string);
         return;
     }
-
-    char *psz_iconv = config_striconv(psz_string);
     dvb_string_clean(p_dvb_string);
-    p_dvb_string->p = dvb_string_set((uint8_t *)psz_iconv, strlen(psz_iconv),
-                                     psz_dvb_charset, &p_dvb_string->i);
+
+    size_t i_iconv;
+    uint8_t *p_iconv = config_striconv(psz_string, psz_charset, &i_iconv);
+    p_dvb_string->p = dvb_string_set(p_iconv, i_iconv, psz_charset,
+                                     &p_dvb_string->i);
+    free(p_iconv);
 }
 
 static bool config_ParseHost( output_config_t *p_config, char *psz_string )
@@ -282,6 +304,11 @@ static bool config_ParseHost( output_config_t *p_config, char *psz_string )
         freeaddrinfo( p_ai );
     }
 
+    const char *psz_charset = psz_dvb_charset;
+    const char *psz_network_name = NULL;
+    const char *psz_service_name = NULL;
+    const char *psz_provider_name = NULL;
+
     while ( (psz_string = strchr( psz_string, '/' )) != NULL )
     {
         *psz_string++ = '\0';
@@ -315,18 +342,14 @@ static bool config_ParseHost( output_config_t *p_config, char *psz_string )
             p_config->i_network_id = strtol( ARG_OPTION("networkid="), NULL, 0 );
         else if ( IS_OPTION("onid=") )
             p_config->i_onid = strtol( ARG_OPTION("onid="), NULL, 0 );
+        else if ( IS_OPTION("charset=")  )
+            psz_charset = ARG_OPTION("charset=");
         else if ( IS_OPTION("networkname=")  )
-        {
-            config_strdvb( &p_config->network_name, ARG_OPTION("networkname=") );
-        }
+            psz_network_name = ARG_OPTION("networkname=");
         else if ( IS_OPTION("srvname=")  )
-        {
-            config_strdvb( &p_config->service_name, ARG_OPTION("srvname=") );
-        }
+            psz_service_name = ARG_OPTION("srvname=");
         else if ( IS_OPTION("srvprovider=") )
-        {
-            config_strdvb( &p_config->provider_name, ARG_OPTION("srvprovider=") );
-        }
+            psz_provider_name = ARG_OPTION("srvprovider=");
         else if ( IS_OPTION("srcaddr=") )
         {
             if ( p_config->i_family != AF_INET ) {
@@ -369,6 +392,14 @@ static bool config_ParseHost( output_config_t *p_config, char *psz_string )
 #undef ARG_OPTION
     }
 
+    if (psz_network_name != NULL)
+        config_strdvb( &p_config->network_name, psz_network_name, psz_charset );
+    if (psz_service_name != NULL)
+        config_strdvb( &p_config->service_name, psz_service_name, psz_charset );
+    if (psz_provider_name != NULL)
+        config_strdvb( &p_config->provider_name, psz_provider_name,
+                       psz_charset );
+
 end:
     i_mtu = p_config->i_family == AF_INET6 ? DEFAULT_IPV6_MTU :
             DEFAULT_IPV4_MTU;
@@ -618,20 +649,25 @@ void usage()
     msg_Raw( NULL, "  -5 --delsys           delivery system" );
     msg_Raw( NULL, "    DVBS|DVBS2|DVBC_ANNEX_A|DVBT|DVBT2|ATSC|ISDBT|DVBC_ANNEX_B(ATSC-C/QAMB) (default guessed)");
     msg_Raw( NULL, "  -f --frequency        frontend frequency" );
-    msg_Raw( NULL, "  -9 --dvb-plp-id <number> Switch PLP of the DVB-T2 transmission (for Russia special)" );
+    msg_Raw( NULL, "  -8 --lnb-type <type>  Set LNB type')" );
+    msg_Raw( NULL, "        universal old-sky (default: universal)");
+    msg_Raw( NULL, "  -9 --dvb-plp-id <number> Switch PLP of the DVB-T2 transmission (default: 0)" );
     msg_Raw( NULL, "  -F --fec-inner        Forward Error Correction (FEC Inner)");
     msg_Raw( NULL, "    DVB-S2 0|12|23|34|35|56|78|89|910|999 (default auto: 999)");
     msg_Raw( NULL, "  -I --inversion        Inversion (-1 auto, 0 off, 1 on)" );
     msg_Raw( NULL, "  -m --modulation       Modulation type" );
     msg_Raw( NULL, "    DVB-C  qpsk|qam_16|qam_32|qam_64|qam_128|qam_256 (default qam_auto)" );
     msg_Raw( NULL, "    DVB-T  qam_16|qam_32|qam_64|qam_128|qam_256 (default qam_auto)" );
-    msg_Raw( NULL, "    DVB-S2 qpsk|psk_8 (default legacy DVB-S)" );
+    msg_Raw( NULL, "    DVB-S2 qpsk|psk_8|apsk_16|apsk_32 (default legacy DVB-S)" );
     msg_Raw( NULL, "  -n --frontend-number <frontend number>" );
     msg_Raw( NULL, "  -p --force-pulse      force 22kHz pulses for high-band selection (DVB-S)" );
     msg_Raw( NULL, "  -P --pilot            DVB-S2 Pilot (-1 auto, 0 off, 1 on)" );
     msg_Raw( NULL, "  -R --rolloff          DVB-S2 Rolloff value" );
     msg_Raw( NULL, "    DVB-S2 35=0.35|25=0.25|20=0.20|0=AUTO (default: 35)" );
-    msg_Raw( NULL, "  -1 --multistream-id   Set stream ID (0-255, default: 0)" );
+    msg_Raw( NULL, "  -1 --multistream-id   Set stream ID (0-2147483648, default: 0)." );
+    msg_Raw( NULL, "     --multistream-id-pls-mode   Set multistream PLS mode (ROOT, GOLD, COMBO, default: ROOT)" );
+    msg_Raw( NULL, "     --multistream-id-pls-code   Set multistream PLS code (0-262143, default: 0)" );
+    msg_Raw( NULL, "     --multistream-id-is-id      Set multistream IS id (0-255, default: 0)" );
     msg_Raw( NULL, "  -K --fec-lp           DVB-T low priority FEC (default auto)" );
     msg_Raw( NULL, "  -G --guard            DVB-T guard interval" );
     msg_Raw( NULL, "    DVB-T  32 (1/32)|16 (1/16)|8 (1/8)|4 (1/4)|-1 (auto, default)" );
@@ -671,8 +707,8 @@ void usage()
     msg_Raw( NULL, "Misc:" );
     msg_Raw( NULL, "  -h --help             display this full help" );
     msg_Raw( NULL, "  -i --priority <RT priority>" );
-    msg_Raw( NULL, "  -j --system-charset   character set used for printing messages (default UTF-8)" );
-    msg_Raw( NULL, "  -J --dvb-charset      character set used in output DVB tables (default UTF-8)" );
+    msg_Raw( NULL, "  -j --system-charset   character set used for printing messages (default UTF-8//IGNORE)" );
+    msg_Raw( NULL, "  -J --dvb-charset      character set used in output DVB tables (default UTF-8//IGNORE)" );
     msg_Raw( NULL, "  -l --logger           use syslog for logging messages instead of stderr" );
     msg_Raw( NULL, "  -g --logger-ident     program name that will be used in syslog messages" );
     msg_Raw( NULL, "  -x --print            print interesting events on stdout in a given format" );
@@ -688,7 +724,7 @@ void usage()
 
 int main( int i_argc, char **pp_argv )
 {
-    const char *psz_network_name = "DVBlast - http://www.videolan.org/projects/dvblast.html";
+    const char *psz_network_name = "DVBlast - videolan.org";
     const char *psz_provider_name = NULL;
     char *psz_dup_config = NULL;
     struct sched_param param;
@@ -704,7 +740,7 @@ int main( int i_argc, char **pp_argv )
         usage();
 
     /*
-     * The only short options left are: 48
+     * The only short options left are: 4
      * Use them wisely.
      */
     static const struct option long_options[] =
@@ -719,6 +755,7 @@ int main( int i_argc, char **pp_argv )
         { "delsys",          required_argument, NULL, '5' },
         { "dvb-plp-id",      required_argument, NULL, '9' },
         { "frequency",       required_argument, NULL, 'f' },
+        { "lnb-type",        required_argument, NULL, '8' },
         { "fec-inner",       required_argument, NULL, 'F' },
         { "rolloff",         required_argument, NULL, 'R' },
         { "symbol-rate",     required_argument, NULL, 's' },
@@ -731,6 +768,9 @@ int main( int i_argc, char **pp_argv )
         { "modulation",      required_argument, NULL, 'm' },
         { "pilot",           required_argument, NULL, 'P' },
         { "multistream-id",  required_argument, NULL, '1' },
+        { "multistream-id-pls-mode",  required_argument, NULL, 0x100001 },
+        { "multistream-id-pls-code",  required_argument, NULL, 0x100002 },
+        { "multistream-id-is-id"   ,  required_argument, NULL, 0x100003 },
         { "fec-lp",          required_argument, NULL, 'K' },
         { "guard",           required_argument, NULL, 'G' },
         { "hierarchy",       required_argument, NULL, 'H' },
@@ -864,6 +904,10 @@ int main( int i_argc, char **pp_argv )
 #endif
             break;
 
+        case '8':
+            psz_lnb_type = optarg;
+            break;
+
         case 'F':
             i_fec = strtol( optarg, NULL, 0 );
             break;
@@ -912,6 +956,36 @@ int main( int i_argc, char **pp_argv )
             i_mis = strtol( optarg, NULL, 0 );
             break;
 
+        case 0x100001: // --multistream-id-pls-mode
+            psz_mis_pls_mode = optarg;
+            if        ( streq( psz_mis_pls_mode, "ROOT" ) ) {
+                i_mis_pls_mode = 0;
+            } else if ( streq( psz_mis_pls_mode, "GOLD" ) ) {
+                i_mis_pls_mode = 1;
+            } else if ( streq( psz_mis_pls_mode, "COMBO" ) ) {
+                i_mis_pls_mode = 2;
+            } else {
+                msg_Err(NULL, "Invalid --multistream-id-pls-mode '%s', valid options are: ROOT GOLD COMBO", optarg);
+                exit(1);
+            }
+            break;
+
+        case 0x100002: // --multistream-id-pls-code
+            i_mis_pls_code = strtol( optarg, NULL, 0 );
+            if ( i_mis_pls_code < 0 || i_mis_pls_code > 262143 ) {
+                msg_Err(NULL, "ERROR: Invalid --multistream-id-pls-code '%s', valid options are: 0-262143", optarg);
+                exit(1);
+            }
+            break;
+
+        case 0x100003: // --multistream-id-is-id
+            i_mis_is_id = strtol( optarg, NULL, 0 );
+            if ( i_mis_is_id < 0 || i_mis_is_id > 255 ) {
+                msg_Err(NULL, "ERROR: Invalid --multistream-id-is-id '%s', valid options are: 0-255", optarg);
+                exit(1);
+            }
+            break;
+
         case 'K':
             i_fec_lp = strtol( optarg, NULL, 0 );
             break;
@@ -1188,8 +1262,8 @@ int main( int i_argc, char **pp_argv )
         config_Free( &config );
     }
 
-    config_strdvb( &network_name, psz_network_name );
-    config_strdvb( &provider_name, psz_provider_name );
+    config_strdvb( &network_name, psz_network_name, psz_dvb_charset );
+    config_strdvb( &provider_name, psz_provider_name, psz_dvb_charset );
 
     /* Set signal handlers */
     signal_watcher_init(&sigint_watcher, event_loop, sighandler, SIGINT);
@@ -1198,6 +1272,26 @@ int main( int i_argc, char **pp_argv )
 
     srand( time(NULL) * getpid() );
 
+    if ( i_mis_pls_mode || i_mis_pls_code || i_mis_is_id )
+    {
+        i_mis = calc_multistream_id( i_mis_pls_mode, i_mis_pls_code, i_mis_is_id );
+        msg_Info( NULL, "Calculating multistream-id using pls-mode: %s (%d) pls-code: %d is-id: %d. Resulting multistream-id: %d (0x%x)",
+            psz_mis_pls_mode, i_mis_pls_mode, i_mis_pls_code, i_mis_is_id, i_mis, i_mis );
+    }
+    else if ( i_mis )
+    {
+        i_mis_pls_mode = (i_mis >> 26) & 0x03;
+        i_mis_pls_code = (i_mis >> 8) & 0x3ffff;
+        i_mis_is_id    = i_mis & 0xff;
+        psz_mis_pls_mode =
+            i_mis_pls_mode == 0 ? "ROOT" :
+            i_mis_pls_mode == 1 ? "GOLD" :
+            i_mis_pls_mode == 2 ? "COMBO" : "UNKNOWN";
+
+        msg_Info( NULL, "Calculated multistream pls-mode: %s (%d) pls-code: %d is-id: %d from multistream-id: %d (0x%x)",
+            psz_mis_pls_mode, i_mis_pls_mode, i_mis_pls_code, i_mis_is_id, i_mis, i_mis );
+    }
+
     demux_Open();
 
     // init the mrtg logfile
@@ -1253,6 +1347,7 @@ int main( int i_argc, char **pp_argv )
 
     comm_Close();
     block_Vacuum();
+    ev_loop_destroy(event_loop);
 
     return EXIT_SUCCESS;
 }
