@@ -2,7 +2,7 @@
  * en50221.c : implementation of the transport, session and applications
  * layers of EN 50 221
  *****************************************************************************
- * Copyright (C) 2004-2005, 2010 VideoLAN
+ * Copyright (C) 2004-2005, 2010, 2015 VideoLAN
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  * Based on code from libdvbci Copyright (C) 2000 Klaus Schmidinger
@@ -21,10 +21,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
+#include "config.h"
+
+#ifdef HAVE_DVB_SUPPORT
 
 #include <stdlib.h>
 #include <stdint.h>
 #include <stdbool.h>
+#include <stddef.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
@@ -39,6 +43,7 @@
 #include <poll.h>
 #include <errno.h>
 #include <time.h>
+#include <ev.h>
 
 /* DVB Card Drivers */
 #include <linux/dvb/version.h>
@@ -73,6 +78,7 @@
 #define CAM_INIT_TIMEOUT 15000000 /* 15 s */
 #undef HLCI_WAIT_CAM_READY
 #define CAPMT_WAIT 100 /* ms */
+#define CA_POLL_PERIOD 100000 /* 100 ms */
 
 typedef struct en50221_msg_t
 {
@@ -107,8 +113,8 @@ typedef struct ci_slot_t
     en50221_msg_t **pp_send_last;
     uint8_t *p;
 
-    /* InitSlot callback, if not 0 */
-    mtime_t i_init_timeout;
+    /* InitSlot timer */
+    struct ev_timer init_watcher;
 
     /* SPDUSend callback, if p_spdu is not NULL */
     /* SessionOpen callback, if not 0 */
@@ -118,6 +124,8 @@ typedef struct ci_slot_t
 int i_ca_handle = 0;
 int i_ca_type = -1;
 
+static struct ev_io cam_watcher;
+static struct ev_timer slot_watcher;
 static int i_nb_slots = 0;
 static ci_slot_t p_slots[MAX_CI_SLOTS];
 static en50221_session_t p_sessions[MAX_SESSIONS];
@@ -133,6 +141,9 @@ static void ResourceManagerOpen( access_t * p_access, int i_session_id );
 static void ApplicationInformationOpen( access_t * p_access, int i_session_id );
 static void ConditionalAccessOpen( access_t * p_access, int i_session_id );
 static void DateTimeOpen( access_t * p_access, int i_session_id );
+static void ResetSlotCb(struct ev_loop *loop, struct ev_timer *w, int revents);
+static void en50221_Read(struct ev_loop *loop, struct ev_io *w, int revents);
+static void en50221_Poll(struct ev_loop *loop, struct ev_timer *w, int revents);
 static void MMIOpen( access_t * p_access, int i_session_id );
 
 /*****************************************************************************
@@ -201,7 +212,7 @@ static uint8_t *SetLength( uint8_t *p_data, int i_length )
  */
 
 #define MAX_TPDU_SIZE  4096
-#define MAX_TPDU_DATA  (MAX_TPDU_SIZE - 4)
+#define MAX_TPDU_DATA  (MAX_TPDU_SIZE - 7)
 
 #define DATA_INDICATOR 0x80
 
@@ -316,9 +327,9 @@ static int TPDUSend( access_t * p_access, uint8_t i_slot, uint8_t i_tag,
 
         if ( i_length )
             memcpy( p, p_content, i_length );
-            i_size = i_length + (p - p_data);
-        }
+        i_size = i_length + (p - p_data);
         break;
+    }
 
     default:
         break;
@@ -384,7 +395,7 @@ static int TPDURecv( access_t * p_access )
     {
     case T_CTC_REPLY:
         p_slot->b_active = true;
-        p_slot->i_init_timeout = 0;
+        ev_timer_stop(event_loop, &p_slot->init_watcher);
         msg_Dbg( p_access, "CI slot %d is active", i_slot );
         break;
 
@@ -571,7 +582,7 @@ static void SessionOpen( access_t * p_access, uint8_t i_slot,
         if ( !p_sessions[i_session_id - 1].i_resource_id )
             break;
     }
-    if ( i_session_id == MAX_SESSIONS )
+    if ( i_session_id > MAX_SESSIONS )
     {
         msg_Err( p_access, "too many sessions !" );
         return;
@@ -629,7 +640,7 @@ static void SessionCreate( access_t * p_access, int i_slot, int i_resource_id )
         if ( !p_sessions[i_session_id - 1].i_resource_id )
             break;
     }
-    if ( i_session_id == MAX_SESSIONS )
+    if ( i_session_id > MAX_SESSIONS )
     {
         msg_Err( p_access, "too many sessions !" );
         return;
@@ -1035,12 +1046,15 @@ static void ApplicationInformationHandle( access_t * p_access, int i_session_id,
             {
             case PRINT_XML:
                 psz_name = dvb_string_xml_escape(psz_name);
-                printf("<STATUS type=\"cam\" status=\"1\" cam_name=\"%s\" cam_type=\"%d\" cam_manufacturer=\"%d\" cam_product=\"%d\" />\n",
-                       psz_name, i_type, i_manufacturer, i_code);
+                fprintf(print_fh, "<STATUS type=\"cam\" status=\"1\" cam_name=\"%s\" cam_type=\"%d\" cam_manufacturer=\"%d\" cam_product=\"%d\" />\n",
+                        psz_name, i_type, i_manufacturer, i_code);
+                break;
+            case PRINT_TEXT:
+                fprintf(print_fh, "CAM name: %s type: %d manufacturer: %d product: %d\n",
+                        psz_name, i_type, i_manufacturer, i_code);
                 break;
             default:
-                printf("CAM: name=%s type=%d manufacturer=%d product=%d\n",
-                       psz_name, i_type, i_manufacturer, i_code);
+                break;
             }
             free(psz_name);
         }
@@ -1406,8 +1420,9 @@ static void ConditionalAccessOpen( access_t * p_access, int i_session_id )
 
 typedef struct
 {
+    int i_session_id;
     int i_interval;
-    mtime_t i_last;
+    struct ev_timer watcher;
 } date_time_t;
 
 /*****************************************************************************
@@ -1444,10 +1459,16 @@ static void DateTimeSend( access_t * p_access, int i_session_id )
 
         APDUSend( p_access, i_session_id, AOT_DATE_TIME, p_response, 7 );
 
-        p_date->i_last = i_wallclock;
+        ev_timer_again(event_loop, &p_date->watcher);
     }
 }
 
+static void _DateTimeSend(struct ev_loop *loop, struct ev_timer *w, int revents)
+{
+    date_time_t *p_date = container_of(w, date_time_t, watcher);
+    DateTimeSend( NULL, p_date->i_session_id );
+}
+
 /*****************************************************************************
  * DateTimeHandle
  *****************************************************************************/
@@ -1475,6 +1496,9 @@ static void DateTimeHandle( access_t * p_access, int i_session_id,
         else
             p_date->i_interval = 0;
 
+        ev_timer_stop(event_loop, &p_date->watcher);
+        ev_timer_set(&p_date->watcher, p_date->i_interval,
+                     p_date->i_interval);
         DateTimeSend( p_access, i_session_id );
         break;
     }
@@ -1483,29 +1507,18 @@ static void DateTimeHandle( access_t * p_access, int i_session_id,
     }
 }
 
-/*****************************************************************************
- * DateTimeManage
- *****************************************************************************/
-static void DateTimeManage( access_t * p_access, int i_session_id )
-{
-    date_time_t *p_date =
-        (date_time_t *)p_sessions[i_session_id - 1].p_sys;
-
-    if ( p_date->i_interval
-          && i_wallclock > p_date->i_last + (mtime_t)p_date->i_interval * 1000000 )
-    {
-        DateTimeSend( p_access, i_session_id );
-    }
-}
-
 /*****************************************************************************
  * DateTimeClose
  *****************************************************************************/
 static void DateTimeClose( access_t * p_access, int i_session_id )
 {
+    date_time_t *p_date =
+        (date_time_t *)p_sessions[i_session_id - 1].p_sys;
+    ev_timer_stop(event_loop, &p_date->watcher);
+
     msg_Dbg( p_access, "closing DateTime session (%d)", i_session_id );
 
-    free( p_sessions[i_session_id - 1].p_sys );
+    free( p_date );
 }
 
 /*****************************************************************************
@@ -1516,11 +1529,16 @@ static void DateTimeOpen( access_t * p_access, int i_session_id )
     msg_Dbg( p_access, "opening DateTime session (%d)", i_session_id );
 
     p_sessions[i_session_id - 1].pf_handle = DateTimeHandle;
-    p_sessions[i_session_id - 1].pf_manage = DateTimeManage;
+    p_sessions[i_session_id - 1].pf_manage = NULL;
     p_sessions[i_session_id - 1].pf_close = DateTimeClose;
     p_sessions[i_session_id - 1].p_sys = malloc(sizeof(date_time_t));
     memset( p_sessions[i_session_id - 1].p_sys, 0, sizeof(date_time_t) );
 
+    date_time_t *p_date =
+        (date_time_t *)p_sessions[i_session_id - 1].p_sys;
+    p_date->i_session_id = i_session_id;
+    ev_timer_init(&p_date->watcher, _DateTimeSend, 0, 0);
+
     DateTimeSend( p_access, i_session_id );
 }
 
@@ -1878,7 +1896,10 @@ static void ResetSlot( int i_slot )
     switch (i_print_type)
     {
     case PRINT_XML:
-        printf("<STATUS type=\"cam\" status=\"0\" />\n");
+        fprintf(print_fh, "<STATUS type=\"cam\" status=\"0\" />\n");
+        break;
+    case PRINT_TEXT:
+        fprintf(print_fh, "CAM none\n");
         break;
     default:
         break;
@@ -1887,7 +1908,9 @@ static void ResetSlot( int i_slot )
     if ( ioctl( i_ca_handle, CA_RESET, 1 << i_slot ) != 0 )
         msg_Err( NULL, "en50221_Poll: couldn't reset slot %d", i_slot );
     p_slot->b_active = false;
-    p_slot->i_init_timeout = mdate() + CAM_INIT_TIMEOUT;
+    ev_timer_init(&p_slot->init_watcher, ResetSlotCb,
+                  CAM_INIT_TIMEOUT / 1000000., 0);
+    ev_timer_start(event_loop, &p_slot->init_watcher);
     p_slot->b_expect_answer = false;
     p_slot->b_mmi_expected = false;
     p_slot->b_mmi_undisplayed = false;
@@ -1921,6 +1944,31 @@ static void ResetSlot( int i_slot )
     }
 }
 
+static void ResetSlotCb(struct ev_loop *loop, struct ev_timer *w, int revents)
+{
+    ci_slot_t *p_slot = container_of(w, ci_slot_t, init_watcher);
+    int i_slot = p_slot - &p_slots[0];
+
+    if ( p_slot->b_active || !p_slot->b_expect_answer )
+        return;
+
+    msg_Warn( NULL, "no answer from CAM, resetting slot %d",
+              i_slot );
+    switch (i_print_type) {
+    case PRINT_XML:
+        fprintf(print_fh,
+                "<EVENT type=\"reset\" cause=\"cam_mute\" />\n");
+        break;
+    case PRINT_TEXT:
+       fprintf(print_fh, "reset cause: cam_mute\n");
+       break;
+    default:
+       break;
+    }
+
+    ResetSlot( i_slot );
+}
+
 
 /*
  * External entry points
@@ -1936,7 +1984,7 @@ void en50221_Init( void )
 
     memset( &caps, 0, sizeof( ca_caps_t ));
 
-    sprintf( psz_tmp, "/dev/dvb/adapter%d/ca0", i_adapter );
+    sprintf( psz_tmp, "/dev/dvb/adapter%d/ca%d", i_adapter, i_canum );
     if( (i_ca_handle = open(psz_tmp, O_RDWR | O_NONBLOCK)) < 0 )
     {
         msg_Warn( NULL, "failed opening CAM device %s (%s)",
@@ -2000,6 +2048,16 @@ void en50221_Init( void )
     i_nb_slots = caps.slot_num;
     memset( p_sessions, 0, sizeof(en50221_session_t) * MAX_SESSIONS );
 
+    if( i_ca_type & CA_CI_LINK )
+    {
+        ev_io_init(&cam_watcher, en50221_Read, i_ca_handle, EV_READ);
+        ev_io_start(event_loop, &cam_watcher);
+
+        ev_timer_init(&slot_watcher, en50221_Poll, CA_POLL_PERIOD / 1000000.,
+                      CA_POLL_PERIOD / 1000000.);
+        ev_timer_start(event_loop, &slot_watcher);
+    }
+
     en50221_Reset();
 }
 
@@ -2102,15 +2160,17 @@ void en50221_Reset( void )
 /*****************************************************************************
  * en50221_Read : Read the CAM for a TPDU
  *****************************************************************************/
-void en50221_Read( void )
+static void en50221_Read(struct ev_loop *loop, struct ev_io *w, int revents)
 {
     TPDURecv( NULL );
+
+    ev_timer_again(event_loop, &slot_watcher);
 }
 
 /*****************************************************************************
  * en50221_Poll : Send a poll TPDU to the CAM
  *****************************************************************************/
-void en50221_Poll( void )
+static void en50221_Poll(struct ev_loop *loop, struct ev_timer *w, int revents)
 {
     int i_slot;
     int i_session_id;
@@ -2137,19 +2197,11 @@ void en50221_Poll( void )
                          i_slot );
                 ResetSlot( i_slot );
             }
-
-            continue;
         }
         else if ( !p_slot->b_active )
         {
             if ( !p_slot->b_expect_answer )
                 InitSlot( NULL, i_slot );
-            else if ( p_slot->i_init_timeout < i_wallclock )
-            {
-                msg_Dbg( NULL, "en50221_Poll: resetting slot %d", i_slot );
-                ResetSlot( i_slot );
-                continue;
-            }
         }
     }
 
@@ -2171,9 +2223,19 @@ void en50221_Poll( void )
         {
             if ( TPDUSend( NULL, i_slot, T_DATA_LAST, NULL, 0 ) != 0 )
             {
-                msg_Err( NULL,
-                         "en50221_Poll: couldn't send TPDU on slot %d, resetting",
-                         i_slot );
+                msg_Warn( NULL, "couldn't send TPDU, resetting slot %d",
+                          i_slot );
+                switch (i_print_type) {
+                case PRINT_XML:
+                    fprintf(print_fh,
+                            "<EVENT type=\"reset\" cause=\"cam_error\" />\n");
+                    break;
+                case PRINT_TEXT:
+                    fprintf(print_fh, "reset cause: cam_error\n");
+                    break;
+                default:
+                    break;
+                }
                 ResetSlot( i_slot );
             }
         }
@@ -2421,3 +2483,15 @@ uint8_t en50221_SendMMIObject( uint8_t *p_buffer, ssize_t i_size )
     msg_Err( NULL, "SendMMIObject when no MMI session is opened !" );
     return RET_ERR;
 }
+
+#else
+#include <inttypes.h>
+
+int i_ca_handle = 0;
+int i_ca_type = -1;
+
+void en50221_AddPMT( uint8_t *p_pmt ) { };
+void en50221_UpdatePMT( uint8_t *p_pmt ) { };
+void en50221_DeletePMT( uint8_t *p_pmt ) { };
+void en50221_Reset( void ) { };
+#endif
